#!/usr/bin/python

##################################
#                                #
# Programmer: Ginny Mak          #
# Location: Montreal, Canada.    #
# Last Updated: April 6, 2000.   #
#                                #
##################################
 
from Tkinter import *
from Dialog import Dialog
from ScrolledText import ScrolledText
import os
import commands


########################################################
#                                                      #
#       This class is a GUI window for smoClassify     #
#                                                      #
########################################################
class ClassifyWindow(Toplevel):
        def start(self):
	    self.title("Classify")
	    self.iconname("classify")
	    
	def makeDisplayBox(self):
	    self.text=ScrolledText(self, height=20, width=50)
	    self.text.pack(anchor=CENTER, fill=BOTH) 
	    self.text.insert(INSERT,
                             "Please fill out all file names.\n", "message")

	def makeWidgets(self):
	    #########################################################
	    #                                                       #
	    # Create a frame for putting in three text labels and   #
	    # and text entries.                                     #
	    #                                                       #
	    #########################################################
	    new = Frame(self) # new frame attaches to self
	    new.pack(anchor=N, expand=YES, fill=BOTH)
	    f1 = Frame(new)   # f1 attaches to new
	    f1.pack(anchor=N, expand=YES, fill=X)
	    Label(f1, text='model file:', relief=RIDGE, width=25)\
                  .pack(side=LEFT)
	    self.e1 = Entry(f1, background="light pink", relief=SUNKEN)
	    self.e1.pack(side=RIGHT, expand=YES, fill=X)
	    self.initial_focus = self.e1
	    self.e1.bind("<Key-Return>", self.gotModelFileName)
		
	    f2 = Frame(new)
	    f2.pack(anchor=CENTER, expand=YES, fill=X)
	    Label(f2, text='test data file:', relief=RIDGE, width=25)\
                  .pack(side=LEFT)
	    self.e2 = Entry(f2, background="light pink", relief=SUNKEN)
	    self.e2.pack(side=RIGHT, expand=YES, fill=X)
	    self.e2.bind("<Key-Return>", self.gotTestFileName)

	    f3 = Frame(new)
	    f3.pack(anchor=S, expand=YES, fill=X)
	    Label(f3, text='prediction file:', relief=RIDGE, width=25)\
                  .pack(side=LEFT)
	    self.e3 = Entry(f3, background="light pink", relief=SUNKEN)
	    self.e3.pack(side=RIGHT, expand=YES, fill=X)
	    self.e3.bind("<Key-Return>", self.gotPredictionFileName)


	def makeToolBar(self):
	    #############################
	    #                           #
	    #  Create three buttons     #
	    #                           #
	    #############################
	    toolbar = Frame(self, cursor='hand2', relief=SUNKEN, bd=2)
	    toolbar.pack(anchor=S,fill=BOTH)
	    Button(toolbar, foreground="white", background="dark blue",\
                   text='Reset', command=self.reset).pack(expand=YES,side=LEFT)
	    Button(toolbar, foreground="white", background="dark blue",\
                   text='Go', command=self.go).pack(expand=YES, side=LEFT)
	    Button(toolbar, foreground="white", background="dark blue",\
                   text='Close', command=self.close).pack(expand=YES, side=LEFT)
	
	
	def __init__ (self):
	    ###############################
	    #                             #
	    # Instantiate a GUI window    #
	    # for smoClassify.            #
	    #                             #
	    ###############################
	    Toplevel.__init__ (self)
	    self.start()
	    self.makeWidgets()
	    self.makeDisplayBox()
	    self.makeToolBar()
	    self.initial_focus = self.e1
	    self.initial_focus.focus_set()
	    self.text.tag_config("error", foreground="red")
	    self.text.tag_config("CprogramOutput", foreground="blue")
	    self.text.tag_config("message", foreground="black")
	    self.grab_set()
	    self.initial_focus.focus_set()
	    self.wait_window()
	
	def close(self):
	    self.destroy()

	def reset(self):
	    self.e1.delete(0, END)
	    self.e2.delete(0, END)
	    self.e3.delete(0, END)
	    self.text.delete(1.0, END)

	def go(self):
	    error = 0
	    if(self.e1.get() == ''):
		self.text.insert(INSERT,"Model file has not been specified.\n",\
                                 "error")
		error = 1
	    if(self.e2.get() == ''):
		self.text.insert(INSERT, "Test file has not been specified\n",\
                                 "error")
		error = 1
	    if(self.e3.get() == ''):
		self.text.insert(INSERT, "Prediction file has not been specified\n",\
                                 "error")	
		error = 1
	    if not error:
	    	cmd = "smoClassify " + self.e1.get() + " "\
                       + self.e2.get() + " " + self.e3.get() 
		output_string=commands.getoutput(cmd);
		self.text.insert(INSERT, output_string+"\n", "CprogramOutput")


	def gotModelFileName(self, event=None):
	    self.e2.focus_set()

	
	def gotTestFileName(self, event=None):
	    self.e3.focus_set()

	def gotPredictionFileName(self, event=None):
	    self.e1.focus_set()    
