#!/usr/bin/python

############################################
#                                          #
# Programmer: Ginny mak                    #
# Location: Montreal, Canada.              #
# Last Updated: April 6, 2000.             #
#                                          #
############################################

from Tkinter import *
from Dialog import Dialog
from ScrolledText import ScrolledText
import os
import commands
import regex

#################################################
#                                               #
#     This class creates a GUI for smoLearn     #
#                                               #
#################################################
class LearnWindow(Toplevel):
    def __init__ (self):
	Toplevel.__init__ (self)
	self.start()
	self.makeWidgets()
	self.makeDisplayBox()
	self.makeToolBar()
	self.text.tag_config("error", foreground="red")
	self.text.tag_config("CprogramOutput", foreground="blue")
	self.text.tag_config("message", foreground="black")
	self.grab_set()
	self.focus_set()
	self.wait_window()

    def start(self):
	self.title("Learning")
	self.iconname("learning")
	self.type = ''
	self.degree = ''
	self.sigma = ''
	self.C = ''
	self.binaryFeature = 0

    def makeDisplayBox(self): # messages to user displayed here
	self.text=ScrolledText(self, height=20, width=50)
	self.text.pack(anchor=CENTER, fill=BOTH)
	self.text.insert(INSERT, "Please choose kernel type first.\n",\
                         "message")
	self.text.insert(INSERT, "Choose Binary Feature ONLY if the features\n"\
                         , "message")
	self.text.insert(INSERT, "of your examples are binary type.\n",\
			 "message")
                   

    def makeWidgets(self):
	#######################################################
	#                                                     #
	#  Create 3 radio buttons, 5 text labels and entries  #
	#                                                     #
	#######################################################
	topFrame = Frame(self)
	topFrame.pack(anchor=N, expand=YES, fill=BOTH)
	kernelButtonFrame = Frame(topFrame)
	kernelButtonFrame.pack(anchor=N, expand=YES, fill=BOTH)
	v = IntVar()
	self.radioButton1 = Radiobutton(kernelButtonFrame, text='Linear',\
                            variable=v, value=0,\
                            command=self.radio1).pack(side=LEFT, expand=YES)
        self.radioButton2 = Radiobutton(kernelButtonFrame, text='Polynomial',\
                            variable=v, value=1,\
                            command=self.radio2).pack(side=LEFT, expand=YES)
	self.radioButton3 = Radiobutton(kernelButtonFrame, text='RBF',\
                            variable=v, value=2,\
                            command=self.radio3).pack(side=LEFT, expand=YES)
	
	labelFrame = Frame(topFrame)
	labelFrame.pack(anchor=CENTER, expand=YES, fill=BOTH)

	###############################
	#                             #
	#   Binary Feature button     #
	#                             #
	###############################
	binaryButtonFrame = Frame(labelFrame)
	binaryButtonFrame.pack(anchor=N, expand=YES, fill=BOTH)
	binVariable = IntVar()
	self.binaryFeature0 = Radiobutton(binaryButtonFrame,\
                             text="Non Binary Feature",\
                             variable=binVariable, value=0,\
			     command=self.nonBinary).pack(side=LEFT, expand=YES)

        self.binaryFeature1 = Radiobutton(binaryButtonFrame,\
                              text="Binary Feature",\
                              variable=binVariable, value=1,\
			      command=self.binary).pack(side=LEFT, expand=YES)

        ###############################
	#                             #
	#   Degree label and entry    #
	#                             #
	###############################	
	f01 = Frame(labelFrame)
	f01.pack(anchor=CENTER, expand=YES, fill=X)
	Label(f01, text='Degree:', relief=RIDGE, width=10).pack(side=LEFT)
	self.e01 = Entry(f01, background="light pink", relief=SUNKEN)
	self.e01.pack(side=RIGHT, expand=YES, fill=X)
	self.e01.bind("<Key-Return>", self.gotDegree)
	
	###############################
	#                             #
	#   Sigma label and entry     #
	#                             #
	###############################
	f02 = Frame(labelFrame)
	f02.pack(anchor=S, expand=YES, fill=X)
	Label(f02, text='Sigma:', relief=RIDGE, width=10).pack(side=LEFT)
	self.e02 = Entry(f02, background="light pink", relief=SUNKEN)
	self.e02.pack(side=RIGHT, expand=YES, fill=X)
	self.e02.bind("<Key-Return>", self.gotSigma)

	###############################
	#                             #
	#   C label and entry         #
	#                             #
	###############################
	textFrame = Frame(topFrame)
	textFrame.pack(anchor=S, expand=YES, fill=X)
	f1 = Frame(textFrame)
	f1.pack(anchor=N, expand=YES, fill=X)
	Label(f1, text='C Parameter:', relief=RIDGE, width=25).pack(side=LEFT)
	self.e1 = Entry(f1, background="light pink", relief=SUNKEN)
	self.e1.pack(side=RIGHT, expand=YES, fill=X)
	self.e1.bind("<Key-Return>", self.gotC)

	#####################################
	#                                   #
	#   Learning file label and entry   #
	#                                   #
	#####################################
	f2 = Frame(textFrame)
	f2.pack(anchor=CENTER, expand=YES, fill=X)
	Label(f2, text='Learning file:', relief=RIDGE, width=25).pack(side=LEFT)
	self.e2 = Entry(f2, background="light pink", relief=SUNKEN)
	self.e2.pack(side=RIGHT, expand=YES, fill=X)
	self.e2.bind("<Key-Return>", self.gotLearn)

	#####################################
	#                                   #
	#   Model file label and entry      #
	#                                   #
	#####################################
	f3 = Frame(textFrame)
	f3.pack(anchor=S, expand=YES, fill=X)
	Label(f3, text='Model file:', relief=RIDGE, width=25).pack(side=LEFT)
	self.e3 = Entry(f3, background="light pink", relief=SUNKEN)
	self.e3.pack(side=RIGHT, expand=YES, fill=X)
	self.e3.bind("<Key-Return>", self.gotModel)


    def makeToolBar(self):
	toolbar = Frame(self, cursor='hand2', relief=SUNKEN, bd=2)
	toolbar.pack(anchor=S,fill=BOTH)
	Button(toolbar, foreground="white", background="dark blue",\
               text='Reset',\
               command=self.reset).pack(expand=YES,side=LEFT)
	Button(toolbar, foreground="white", background="dark blue", text='Go',\
               command=self.go).pack(expand=YES, side=LEFT)
	Button(toolbar, foreground="white", background="dark blue",\
               text='Close',\
               command=self.close).pack(expand=YES, side=LEFT) 

                                       
    def close(self):
	self.destroy()

    def reset(self):
	self.e01.delete(0, END)
	self.e02.delete(0, END)
	self.e1.delete(0, END)
	self.e2.delete(0, END)
	self.e3.delete(0, END)
	self.text.delete(1.0, END)

    def go(self):
	self.degree = self.e01.get()
	self.sigma = self.e02.get()
	self.C = self.e1.get()

	if self.type == '':
	    self.text.insert(INSERT, "kernel type has not been chosen\n",\
                             "error")
	else:
	    ################################################################
	    #                                                              #
	    # Do type checking of inputs of user. Make sure all necessary  #
	    # are entered before calling smoLearn.                         #
	    #                                                              #
	    ################################################################
	    if self.type == '0':
		if self.e1.get() == '':
		    self.text.insert(INSERT, "Need to specify C parameter\n",\
                                     "error")
		elif regex.search("[^0123456789.]", self.e1.get()) != -1 :
		    self.text.insert(INSERT, "C has to be a number > 0\n",\
                                     "error")
		elif self.e2.get() == '':
		    self.text.insert(INSERT,\
                               "Learning file name has not been specified\n",\
                               "error")
		elif self.e3.get() == '':	
			self.text.insert(INSERT,\
                                "Model file name has not specified\n",\
                                "error")	
		else:
		    if self.binaryFeature == 0:
			cmd = "smoLearn " + "-t " + self.type + " -c "\
                          + self.C + " " + self.e2.get() + " " + self.e3.get()
		    else:
			cmd = "smoLearn " + "-t " + self.type + " -c "\
                              + self.C + " -b 1 " + self.e2.get()\
                              + " " + self.e3.get()
		    output_string = commands.getoutput(cmd)
		    self.text.insert(INSERT, output_string+"\n","CprogramOutput")

	    elif self.type == '1':
		if self.degree == '':
		    self.text.insert(INSERT,\
                           "Need to specify degree for polynomial kernel\n",\
                           "error")
		elif regex.search("[^0123456789]", self.degree) != -1 :	
		    self.text.insert(INSERT,\
                            "degree has to be an integer > 0\n",\
                            "error")
		elif self.e1.get() == '':
		    self.text.insert(INSERT, "Need to specify C parameter\n",\
                                     "error") 
		elif regex.search("[^0123456789.]", self.e1.get()) != -1 : 
		    self.text.insert(INSERT, "C has to be a number > 0\n",\
                                     "error")
		elif self.e2.get() == '':
		    self.text.insert(INSERT,\
                                "Learning file name has not been specified\n",\
                                "error")
		elif self.e3.get() == '':	
		    self.text.insert(INSERT,\
                                     "Model file name has not specified\n",\
                                     "error")		
		else:
		    if self.binaryFeature == 0:
			cmd = "smoLearn " + "-t " + self.type + " -c " + self.C\
                           + " -d " + self.degree + " " + self.e2.get() + " " \
                           + self.e3.get()
		    else:
			cmd = "smoLearn " + "-t " + self.type + " -c " + self.C\
                           + " -d " + self.degree + " -b 1 "\
                           +  self.e2.get() + " " + self.e3.get()
		    output_string = commands.getoutput(cmd)
		    self.text.insert(INSERT,output_string+"\n","CprogramOutput")	

	    elif self.type == '2':
		if self.e02.get() == '':
		    self.text.insert(INSERT,\
                                     "Need to specify sigma for rbf kernel\n",\
                                     "error")
		elif regex.search("[^0123456789.]", self.e02.get()) != -1 : 
		    self.text.insert(INSERT, "Sigma has to be a number > 0\n",\
                                     "error")
		elif self.e1.get() == '':
		    self.text.insert(INSERT, "Need to specify C parameter\n",\
				     "error") 
		elif regex.search("[^0123456789.]", self.e1.get()) != -1 :
		    self.text.insert(INSERT, "C has to be a number > 0\n",\
                                     "error")
		elif self.e2.get() == '':
		    self.text.insert(INSERT,\
                               "Learning file name has not been specified\n",\
                               "error")
		elif self.e3.get() == '':	
		    self.text.insert(INSERT,\
                                     "Model file name has not specified\n",\
                                     "error")		
		else:
		    if self.binaryFeature == 0:
			cmd = "smoLearn " + "-t " + self.type + " -c " + self.C\
			      + " -v " + self.sigma + " " + self.e2.get() + " "\
			      + self.e3.get()
		    else:
			cmd = "smoLearn " + "-t " + self.type + " -c " + self.C\
			      + " -v " + self.sigma + " -b 1 "\
                              + self.e2.get() + " " + self.e3.get()
		    output_string = commands.getoutput(cmd)
		    self.text.insert(INSERT, output_string+"\n","CprogramOutput")


    def radio1(self):
	#####################################################
	#                                                   #
	# User selects linear kernel. Prompt user to enter  #
	# C parameter, learning file and model file names.  #
	#                                                   #
	#####################################################
	self.type = '0'
	self.e1.focus_set()
	self.text.insert(INSERT,\
                         "Please input C parameter, learning file name\n",\
                         "message")
	self.text.insert(INSERT, "and model file name.\n", "message")
	self.text.insert(INSERT, "C has to be > 0\n", "message")

    def radio2(self):
	########################################################
	#                                                      #
	# User selects polynomial kernel. Prompt user to       #
	# enter degree, C, learning file and model file names. #
	#                                                      #
	########################################################
	self.type = '1'
	self.e01.focus_set()
	self.text.insert(INSERT,"Please input degree, C, learning file name\n",\
                         "message")
	self.text.insert(INSERT, "and model file name.\n", "message")
	self.text.insert(INSERT, "degree and C have to be > 0\n", "message")

    def radio3(self):
	########################################################
	#                                                      #
	# User selects rbf kernel. Prompt user to enter sigma, #
	# C, learning file and model file name.                #
	#                                                      #
	########################################################
	self.type = '2'
	self.e02.focus_set()
	self.text.insert(INSERT, "Please input sigma, C, learning file name\n",\
                         "message")
	self.text.insert(INSERT, "and model file name.\n", "message")
	self.text.insert(INSERT, "sigma and C have to be > 0\n", "message")

    def binary(self):
	########################################################
	#                                                      #
	# User's training examples have binary valued features #
	#                                                      #
	########################################################
	self.binaryFeature = 1

	
    def nonBinary(self):
	########################################################
        #                                                      #
        # User's training examples have non binary valued      #
        # features.                                            #
        #                                                      #
        ########################################################      
	self.binaryFeature = 0

    def gotDegree(self,event=None):
	################################
	#                              #
	#    Move cursor to C entry    #
	#                              #
	################################
	self.e1.focus_set()

    def gotSigma(self, event=None):
	################################
	#                              #
	#    Move cursor to C entry    #
	#                              #
	################################
	self.e1.focus_set() 

    def gotC(self,event=None):
	################################################
	#                                              #
	#    Move cursor to learning file name entry   #
	#                                              #
	################################################
	self.e2.focus_set()

	
    def gotLearn(self, event=None):
	################################################
	#                                              #
	#      Move cursor to model file name entry    #
	#                                              #
	################################################	
	self.e3.focus_set()


    def gotModel(self, event=None):
	###################################################
	#                                                 #
	# If kernel is linear, move cursor to C parameter #
	# entry.                                          #
	# If kernel is polynomial, move cursor to degree  #
	# entry.                                          #
	# If kernel is rbf, move cursor to sigma entry    #
	#                                                 #
	###################################################
	if self.type == '0':
	    self.e1.focus_set()
	if self.type == '1':
	    self.e01.focus_set()
	if self.type == '2':
	    self.e02.focus_set()


if __name__ == '__main__':
    LearnWindow().mainloop()

