/***************************   smoClassify.c  ************************

 Purpose: Program main function.

 Notes: This program is composed of the following modules:
        
        smoClassify.c - main program module.
        initialize.c  - Initialize all data structures required to do
                        classification by reading the information from
                        the model file and the test data file.
        classify.c    - Do the classification calculations and
                        write the result to the prediction file.
			
 Programmer: Ginny Mak
 Location: Montreal, Canada.
 Last Updated: April 6, 2000. 

**/

#include <stdio.h>
#include "initialize.h"
#include "classify.h"


int main(int argc, char *argv[])
{
  FILE *modelIn, *dataIn, *out;
  double startTime;

  /*************************************
   *                                   *
   * Check on command line usage       * 
   *                                   *
   *************************************/                                   
  if( argc != 4 ) {
    fprintf( stderr, "Usage: Command line\n" );
    exit(1); 
  }
  else {
    if(( modelIn = fopen( argv[1], "r" ) ) == NULL ) {
      fprintf( stderr, "Can't open %s\n", argv[1] );
      exit(2); 
    }

    if(( dataIn = fopen( argv[2], "r" ) ) == NULL ) {
      fprintf( stderr, "Can't open %s\n", argv[2] );
      exit(2); 
    }

    if(( out = fopen( argv[3], "w" ) ) == NULL ) {
      fprintf( stderr, "Can't open %s\n", argv[3] );
      exit(2); 
    }
  }

  /***********************
   *                     *
   * Read model file     *
   *                     *
   ***********************/                     
  if( ! readModel( modelIn ) ) {
    fprintf( stderr, "Error in reading model file %s\n", argv[1] ); 
    exit (3);
  }
  else 
    fclose( modelIn );
  printf("Finish reading model file\n");
  
  /***********************                     
   *                     *
   * Read test data file *
   *                     *
   ***********************/                     
  if( !readData( dataIn )) {
    printf("Error reading data file\n");
    exit(4);
  }    
  fclose( dataIn );
  printf("Finish reading test data file\n");
 
  /***********************
   *                     *
   * Start classfying.   *
   *                     *
   ***********************/                                     
  if ( writeResult( out ) ) 
    printf("Classification is completed\n");
  else
    fprintf( stderr, "Classification process failed\n");

  fclose( out );
  return 0;
}
